@@ -32,6 +32,7 @@
 #include <stdbool.h>
 #include <string.h>
 #include <unistd.h>
+#include <fcntl.h>
 #include <limits.h>
 #include <inttypes.h>
 #include <libgen.h>
@@ -39,6 +40,7 @@
 #include <time.h>
 #include <math.h>
 #include <signal.h>
+#include <sys/ioctl.h>
 #include <sys/stat.h>
 #include <sys/types.h>
 #if defined(__linux__)
@@ -91,17 +93,19 @@ size_t stress_get_pagesize(void)
  */
 int32_t stress_get_processors_online(void)
 {
-	static uint32_t processors_online = 0;
+	static int32_t processors_online = 0;
 
 	if (processors_online > 0)
 		return processors_online;
 
 #ifdef _SC_NPROCESSORS_ONLN
-	processors_online = (uint32_t)sysconf(_SC_NPROCESSORS_ONLN);
-	return processors_online;
+	processors_online = (int32_t)sysconf(_SC_NPROCESSORS_ONLN);
+	if (processors_online < 0)
+		processors_online = 1;
 #else
-	return -1;
+	processors_online = 1;
 #endif
+	return processors_online;
 }
 
 /*
@@ -110,17 +114,19 @@ int32_t stress_get_processors_online(void)
  */
 int32_t stress_get_processors_configured(void)
 {
-	static uint32_t processors_configured = 0;
+	static int32_t processors_configured = 0;
 
 	if (processors_configured > 0)
 		return processors_configured;
 
 #ifdef _SC_NPROCESSORS_CONF
-	processors_configured = sysconf(_SC_NPROCESSORS_CONF);
-	return processors_configured;
+	processors_configured = (int32_t)sysconf(_SC_NPROCESSORS_CONF);
+	if (processors_configured < 0)
+		processors_configured = stress_get_processors_online();
 #else
-	return -1;
+	processors_configured = 1;
 #endif
+	return processors_configured;
 }
 
 /*
@@ -178,6 +184,25 @@ void stress_get_memlimits(
 #endif
 }
 
+/*
+ *  stress_set_nonblock()
+ *	try to make fd non-blocking
+ */
+int stress_set_nonblock(const int fd)
+{
+	int flags;
+#if defined(O_NONBLOCK)
+
+	if ((flags = fcntl(fd, F_GETFL, 0)) < 0)
+		flags = 0;
+	return fcntl(fd, F_SETFL, O_NONBLOCK | flags);
+#else
+	flags = 1;
+	return ioctl(fd, FIOBIO, &flags);
+#endif
+}
+
+
 /*
  *  stress_get_load_avg()
  *	get load average
@@ -240,6 +265,11 @@ int stress_process_dumpable(const bool dumpable)
 {
 	char path[PATH_MAX];
 	int fd, rc = 0;
+#if defined(RLIMIT_CORE)
+	struct rlimit lim = { 0, 0 };
+
+	(void)setrlimit(RLIMIT_CORE, &lim);
+#endif
 
 #if defined(__linux__) && defined(PR_SET_DUMPABLE)
 	(void)prctl(PR_SET_DUMPABLE,
@@ -264,7 +294,7 @@ int stress_process_dumpable(const bool dumpable)
 void stress_set_timer_slack_ns(const char *optarg)
 {
 #if defined(__linux__) && defined(PRCTL_TIMER_SLACK)
-	timer_slack = get_unsigned_long(optarg);
+	timer_slack = get_uint32(optarg);
 #else
 	(void)optarg;
 #endif
@@ -568,8 +598,9 @@ int stress_cache_alloc(const char *name)
 #else
 	cpu_caches = get_all_cpu_cache_details();
 	if (!cpu_caches) {
-		pr_inf(stderr, "%s: using built-in defaults as unable to "
-			"determine cache details\n", name);
+		if (warn_once(WARN_ONCE_CACHE_DEFAULT))
+			pr_inf(stderr, "%s: using built-in defaults as unable to "
+				"determine cache details\n", name);
 		shared->mem_cache_size = MEM_CACHE_SIZE;
 		goto init_done;
 	}
@@ -577,16 +608,18 @@ int stress_cache_alloc(const char *name)
 	max_cache_level = get_max_cache_level(cpu_caches);
 
 	if (shared->mem_cache_level > max_cache_level) {
-		pr_dbg(stderr, "%s: reducing cache level from L%d (too high) "
-			"to L%d\n", name,
-			shared->mem_cache_level, max_cache_level);
+		if (warn_once(WARN_ONCE_CACHE_REDUCED))
+			pr_dbg(stderr, "%s: reducing cache level from L%d (too high) "
+				"to L%d\n", name,
+				shared->mem_cache_level, max_cache_level);
 		shared->mem_cache_level = max_cache_level;
 	}
 
 	cache = get_cpu_cache(cpu_caches, shared->mem_cache_level);
 	if (!cache) {
-		pr_inf(stderr, "%s: using built-in defaults as no suitable "
-			"cache found\n", name);
+		if (warn_once(WARN_ONCE_CACHE_NONE))
+			pr_inf(stderr, "%s: using built-in defaults as no suitable "
+				"cache found\n", name);
 		shared->mem_cache_size = MEM_CACHE_SIZE;
 		goto init_done;
 	}
@@ -595,9 +628,10 @@ int stress_cache_alloc(const char *name)
 		uint64_t way_size;
 
 		if (shared->mem_cache_ways > cache->ways) {
-			pr_inf(stderr, "%s: cache way value too high - "
-				"defaulting to %d (the maximum)\n",
-				name, cache->ways);
+			if (warn_once(WARN_ONCE_CACHE_WAY))
+				pr_inf(stderr, "%s: cache way value too high - "
+					"defaulting to %d (the maximum)\n",
+					name, cache->ways);
 			shared->mem_cache_ways = cache->ways;
 		}
 
@@ -611,8 +645,9 @@ int stress_cache_alloc(const char *name)
 	}
 
 	if (!shared->mem_cache_size) {
-		pr_inf(stderr, "%s: using built-in defaults as unable to "
-			"determine cache size\n", name);
+		if (warn_once(WARN_ONCE_CACHE_DEFAULT))
+			pr_inf(stderr, "%s: using built-in defaults as "
+				"unable to determine cache size\n", name);
 		shared->mem_cache_size = MEM_CACHE_SIZE;
 	}
 init_done:
@@ -624,8 +659,9 @@ init_done:
 			name);
 		return -1;
 	}
-	pr_inf(stderr, "%s: default cache size: %" PRIu64 "K\n",
-		name, shared->mem_cache_size / 1024);
+	if (warn_once(WARN_ONCE_CACHE_SIZE))
+		pr_inf(stderr, "%s: default cache size: %" PRIu64 "K\n",
+			name, shared->mem_cache_size / 1024);
 
 	return 0;
 }
