@@ -33,7 +33,7 @@
 
 #include "stress-ng.h"
 
-#if !defined(__gnu_hurd__)
+#if !defined(__gnu_hurd__) && !defined(__minix__)
 static const int madvise_options[] = {
 #ifdef MADV_NORMAL
 	MADV_NORMAL,
@@ -100,7 +100,7 @@ static const int madvise_options[] = {
  */
 int madvise_random(void *addr, const size_t length)
 {
-#if !defined(__gnu_hurd__)
+#if !defined(__gnu_hurd__) && !defined(__minix__)
 	if (opt_flags & OPT_FLAGS_MMAP_MADVISE) {
 		int i = (mwc32() >> 7) % SIZEOF_ARRAY(madvise_options);
 
