@@ -49,7 +49,7 @@ void stress_set_aio_linux_requests(const char *optarg)
 	uint32_t aio_linux_requests;
 
 	set_aio_linux_requests = true;
-	aio_linux_requests = get_unsigned_long(optarg);
+	aio_linux_requests = get_uint32(optarg);
 	check_range("aiol-requests", aio_linux_requests,
 		MIN_AIO_LINUX_REQUESTS, MAX_AIO_LINUX_REQUESTS);
 	opt_aio_linux_requests = aio_linux_requests;
@@ -71,10 +71,10 @@ static inline void aio_linux_fill_buffer(
 }
 
 /*
- *  stress_aio_linux
+ *  stress_aiol
  *	stress asynchronous I/O using the linux specific aio ABI
  */
-int stress_aio_linux(
+int stress_aiol(
 	uint64_t *const counter,
 	const uint32_t instance,
 	const uint64_t max_ops,
