@@ -24,14 +24,31 @@
  */
 #define _GNU_SOURCE
 
+#include "stress-ng.h"
+
 #include <stdio.h>
 #include <stdlib.h>
 #include <stdint.h>
-#if defined(_POSIX_PRIORITY_SCHEDULING) || defined(__linux__)
+#include <stdbool.h>
+#if defined(__linux__)
+#include <sys/syscall.h>
+#ifndef ICACHE
+#define	ICACHE	(1 << 0)
+#endif
+#ifndef DCACHE
+#define DCACHE	(1 << 1)
+#endif
+#if defined(_POSIX_PRIORITY_SCHEDULING)
 #include <sched.h>
 #endif
+#endif
 
-#include "stress-ng.h"
+#if defined(__linux__) && defined(__NR_cacheflush)
+static inline int sys_cacheflush(char *addr, int nbytes, int cache)
+{
+	return (int)syscall(__NR_cacheflush, addr, nbytes, cache);
+}
+#endif
 
 /* The compiler optimises out the unused cache flush and mfence calls */
 #define CACHE_WRITE(flag)						\
@@ -67,7 +84,8 @@ int stress_cache(
 	cpu_set_t mask;
 	uint32_t cpu = 0;
 	const uint32_t cpus = stress_get_processors_configured();
-	int pinned = false;
+	cpu_set_t proc_mask;
+	bool pinned = false;
 #endif
 	uint32_t total = 0;
 	int ret = EXIT_SUCCESS;
@@ -78,6 +96,20 @@ int stress_cache(
 		pr_dbg(stderr, "%s: using cache buffer size of %" PRIu64 "K\n",
 			name, mem_cache_size / 1024);
 
+#if defined(__linux__)
+	if (sched_getaffinity(0, sizeof(proc_mask), &proc_mask) < 0)
+		pinned = true;
+	else
+		if (!CPU_COUNT(&proc_mask))
+			pinned = true;
+
+	if (pinned) {
+		pr_inf(stderr, "%s: can't get sched affinity, pinning to "
+			"CPU %d (instance %" PRIu32 ")\n",
+			name, sched_getcpu(), pinned);
+	}
+#endif
+
 	do {
 		uint64_t i = mwc64() % mem_cache_size;
 		uint64_t r = mwc64();
@@ -136,15 +168,17 @@ int stress_cache(
 
 			cpu = (int32_t)current;
 		} else {
-			cpu = (opt_flags & OPT_FLAGS_AFFINITY_RAND) ?
-				(mwc32() >> 4) : cpu + 1;
-			cpu %= cpus;
+			do {
+				cpu = (opt_flags & OPT_FLAGS_AFFINITY_RAND) ?
+					(mwc32() >> 4) : cpu + 1;
+				cpu %= cpus;
+			} while (!(CPU_ISSET(cpu, &proc_mask)));
 		}
 
 		if (!(opt_flags & OPT_FLAGS_CACHE_NOAFF) || !pinned) {
 			CPU_ZERO(&mask);
 			CPU_SET(cpu, &mask);
-			sched_setaffinity(0, sizeof(mask), &mask);
+			(void)sched_setaffinity(0, sizeof(mask), &mask);
 
 			if ((opt_flags & OPT_FLAGS_CACHE_NOAFF)) {
 				/* Don't continually set the affinity */
@@ -152,6 +186,10 @@ int stress_cache(
 			}
 
 		}
+#endif
+#if defined(__linux__) && defined(__NR_cacheflush)
+		sys_cacheflush((char *)stress_cache, 8192, ICACHE);
+		sys_cacheflush((char *)mem_cache, (int)mem_cache_size, DCACHE);
 #endif
 		(*counter)++;
 	} while (opt_do_run && (!max_ops || *counter < max_ops));
