@@ -60,6 +60,10 @@
 #define powl	pow
 #endif
 
+#if defined(__minix__)
+#define cabsl	cabs
+#endif
+
 /*
  *  the CPU stress test has different classes of cpu stressor
  */
@@ -1453,7 +1457,7 @@ static void HOT OPTIMIZE3 stress_cpu_correlate(const char *name)
 static void HOT OPTIMIZE3 stress_cpu_sieve(const char *name)
 {
 	const uint32_t nsqrt = sqrt(SIEVE_SIZE);
-	uint32_t sieve[(SIEVE_SIZE + 31) / 32];
+	static uint32_t sieve[(SIEVE_SIZE + 31) / 32];
 	uint32_t i, j;
 
 	memset(sieve, 0xff, sizeof(sieve));
