@@ -105,7 +105,8 @@ int stress_dir(
 
 abort:
 	/* force unlink of all files */
-	pr_tidy(stderr, "%s: removing %" PRIu32 " directories\n", name, DEFAULT_DIRS);
+	pr_tidy(stderr, "%s: removing %" PRIu32 " directories\n",
+		name, DEFAULT_DIRS);
 	stress_dir_tidy(DEFAULT_DIRS, name, pid, instance);
 	(void)stress_temp_dir_rm(name, pid, instance);
 
