@@ -118,7 +118,9 @@ int stress_fallocate(
 			if (fstat(fd, &buf) < 0)
 				pr_fail(stderr, "%s: fstat on file failed", name);
 			else if (buf.st_size != opt_fallocate_bytes)
-				pr_fail(stderr, "%s: file size %jd does not match size the expected file size of %jd\n",
+				pr_fail(stderr, "%s: file size %jd does not "
+					"match size the expected file "
+					"size of %jd\n",
 					name, (intmax_t)buf.st_size,
 					(intmax_t)opt_fallocate_bytes);
 		}
@@ -135,9 +137,10 @@ int stress_fallocate(
 			if (fstat(fd, &buf) < 0)
 				pr_fail(stderr, "%s: fstat on file failed", name);
 			else if (buf.st_size != (off_t)0)
-				pr_fail(stderr, "%s: file size %jd does not match size the expected file size of 0\n",
+				pr_fail(stderr, "%s: file size %jd does not "
+					"match size the expected file size "
+					"of 0\n",
 					name, (intmax_t)buf.st_size);
-
 		}
 
 		if (ftruncate(fd, opt_fallocate_bytes) < 0)
