@@ -28,6 +28,7 @@
 #include <stdlib.h>
 #include <stdint.h>
 #include <errno.h>
+#include <setjmp.h>
 #include <sys/mman.h>
 #include <sys/types.h>
 #include <sys/stat.h>
@@ -37,6 +38,21 @@
 
 #include "stress-ng.h"
 
+static sigjmp_buf jmp_env;
+static volatile bool do_jmp = true;
+
+/*
+ *  stress_segvhandler()
+ *	SEGV handler
+ */
+static void MLOCKED stress_segvhandler(int dummy)
+{
+	(void)dummy;
+
+	if (do_jmp)
+		siglongjmp(jmp_env, 1);		/* Ugly, bounce back */
+}
+
 /*
  *  stress_fault()
  *	stress min and max page faulting
@@ -49,7 +65,7 @@ int stress_fault(
 {
 	struct rusage usage;
 	char filename[PATH_MAX];
-	int ret, i = 0;
+	int ret, i;
 	const pid_t pid = getpid();
 
 	ret = stress_temp_dir_mk(name, pid, instance);
@@ -60,10 +76,23 @@ int stress_fault(
 		name, pid, instance, mwc32());
 	(void)umask(0077);
 
+	i = 0;
+
+	if (stress_sighandler(name, SIGSEGV, stress_segvhandler, NULL) < 0)
+		return EXIT_FAILURE;
+
 	do {
 		char *ptr;
 		int fd;
 
+		ret = sigsetjmp(jmp_env, 1);
+		if (ret) {
+			do_jmp = false;
+			pr_err(stderr, "%s: unexpected segmentation fault\n",
+				name);
+			break;
+		}
+
 		fd = open(filename, O_RDWR | O_CREAT, S_IRUSR | S_IWUSR);
 		if (fd < 0) {
 			if ((errno == ENOSPC) || (errno == ENOMEM))
@@ -79,7 +108,8 @@ int stress_fault(
 				continue;	/* Try again */
 			}
 			(void)close(fd);
-			pr_err(stderr, "%s: posix_fallocate failed: errno=%d (%s)\n",
+			pr_err(stderr, "%s: posix_fallocate failed: "
+				"errno=%d (%s)\n",
 				name, errno, strerror(errno));
 			break;
 		}
@@ -88,7 +118,8 @@ int stress_fault(
 			char buffer[1];
 
 redo:
-			if (opt_do_run && (write(fd, buffer, sizeof(buffer)) < 0)) {
+			if (opt_do_run &&
+			    (write(fd, buffer, sizeof(buffer)) < 0)) {
 				if ((errno == EAGAIN) || (errno == EINTR))
 					goto redo;
 				if (errno == ENOSPC) {
@@ -96,12 +127,21 @@ redo:
 					continue;
 				}
 				(void)close(fd);
-				pr_err(stderr, "%s: write failed: errno=%d (%s)\n",
+				pr_err(stderr, "%s: write failed: "
+					"errno=%d (%s)\n",
 					name, errno, strerror(errno));
 				break;
 			}
 		}
 #endif
+		ret = sigsetjmp(jmp_env, 1);
+		if (ret) {
+			if (!opt_do_run || (max_ops && *counter >= max_ops))
+				do_jmp = false;
+			if (fd != -1)
+				(void)close(fd);
+			goto next;
+		}
 
 		/*
 		 * Removing file here causes major fault when we touch
@@ -113,6 +153,7 @@ redo:
 		ptr = mmap(NULL, 1, PROT_READ | PROT_WRITE,
 			MAP_SHARED, fd, 0);
 		(void)close(fd);
+		fd = -1;
 
 		if (ptr == MAP_FAILED) {
 			pr_err(stderr, "%s: mmap failed: errno=%d (%s)\n",
@@ -128,6 +169,7 @@ redo:
 			break;
 		}
 
+next:
 		/* Remove file on-non major fault case */
 		if (!(i & 1))
 			(void)unlink(filename);
