@@ -117,13 +117,22 @@ static const int priorities[] = {
 #endif
 };
 
-#if defined(__linux__)
+#if defined(__linux__) && defined(__NR_gettid)
 static inline int sys_gettid(void)
 {
         return syscall(SYS_gettid);
 }
 #endif
 
+#if defined(__linux__) && defined(__NR_getcpu)
+static long sys_getcpu(
+        unsigned *cpu,
+        unsigned *node,
+        void *tcache)
+{
+	return syscall(__NR_getcpu, cpu, node, tcache);
+}
+#endif
 
 /*
  *  stress on get*() calls
@@ -149,7 +158,9 @@ int stress_get(
 		struct utsname utsbuf;
 		struct timex timexbuf;
 #endif
+#if !defined(__minix__)
 		const pid_t mypid = getpid();
+#endif
 		int ret;
 		size_t i;
 		struct timeval tv;
@@ -184,11 +195,15 @@ int stress_get(
 				name, errno, strerror(errno));
 		check_do_run();
 
+#if !defined(__minix__)
 		(void)getpgrp();
 		check_do_run();
+#endif
 
+#if !defined(__minix__)
 		(void)getpgid(mypid);
 		check_do_run();
+#endif
 
 		for (i = 0; i < SIZEOF_ARRAY(priorities); i++) {
 			errno = 0;
@@ -223,7 +238,7 @@ int stress_get(
 			check_do_run();
 		}
 
-#if defined(__linux__)
+#if defined(__linux__) && NEED_GLIBC(2,13,0)
 		for (i = 0; i < SIZEOF_ARRAY(rlimits); i++) {
 			struct rlimit rlim[2];
 
@@ -264,11 +279,22 @@ int stress_get(
 		check_do_run();
 #endif
 
-#if defined(__linux__)
+#if defined(__linux__) && defined(__NR_gettid)
 		(void)sys_gettid();
 		check_do_run();
 #endif
 
+#if defined(__linux__) && defined(__NR_getcpu)
+		{
+			unsigned cpu, node;
+
+			(void)sys_getcpu(&cpu, &node, NULL);
+			(void)sys_getcpu(NULL, &node, NULL);
+			(void)sys_getcpu(&cpu, NULL, NULL);
+			(void)sys_getcpu(NULL, NULL, NULL);
+		}
+#endif
+
 		t = time(NULL);
 		if (verify && (t == (time_t)-1))
 			pr_fail(stderr, "%s: time failed, errno=%d (%s)\n",
