@@ -39,9 +39,20 @@
 #include <signal.h>
 #include <sys/types.h>
 #include <sys/wait.h>
+#include <fcntl.h>
 
 #define MAX_MEM_FDS 	(256)
-#define MEM_PAGES	(1024)
+
+static size_t opt_memfd_bytes = DEFAULT_MEMFD_BYTES;
+static bool set_memfd_bytes;
+
+void stress_set_memfd_bytes(const char *optarg)
+{
+	set_memfd_bytes = true;
+	opt_memfd_bytes = (size_t)get_uint64_byte(optarg);
+	check_range("memfd-bytes", opt_memfd_bytes,
+		MIN_MEMFD_BYTES, MAX_MEMFD_BYTES);
+}
 
 /*
  *  Ugly hack until glibc defines this
@@ -67,9 +78,14 @@ static void stress_memfd_allocs(
 	int fds[MAX_MEM_FDS];
 	void *maps[MAX_MEM_FDS];
 	size_t i;
-	const size_t size = stress_get_pagesize() * MEM_PAGES;
+	const size_t page_size = stress_get_pagesize();
+	const size_t min_size = 2 * page_size;
+	size_t size = opt_memfd_bytes / MAX_MEM_FDS;
 	const pid_t pid = getpid();
 
+	if (size < min_size)
+		size = min_size;
+
 	do {
 		for (i = 0; i < MAX_MEM_FDS; i++) {
 			fds[i] = -1;
@@ -97,11 +113,14 @@ static void stress_memfd_allocs(
 					goto clean;
 				}
 			}
+			if (!opt_do_run)
+				goto clean;
 		}
-	
+
 		for (i = 0; i < MAX_MEM_FDS; i++) {
 			if (fds[i] >= 0) {
 				ssize_t ret;
+				size_t whence;
 
 				if (!opt_do_run)
 					break;
@@ -126,8 +145,62 @@ static void stress_memfd_allocs(
 					MAP_FILE | MAP_SHARED | MAP_POPULATE,
 					fds[i], 0);
 				mincore_touch_pages(maps[i], size);
+				madvise_random(maps[i], size);
+
+#if defined(FALLOC_FL_PUNCH_HOLE) && defined(FALLOC_FL_KEEP_SIZE)
+				/*
+				 *  ..and punch a hole
+				 */
+				whence = (mwc32() % size) & ~(page_size - 1);
+				ret = fallocate(fds[i], FALLOC_FL_PUNCH_HOLE |
+					FALLOC_FL_KEEP_SIZE, whence, page_size);
+				(void)ret;
+#endif
 			}
+			if (!opt_do_run)
+				goto clean;
 		}
+
+		for (i = 0; i < MAX_MEM_FDS; i++) {
+#if defined(SEEK_SET)
+			if (lseek(fds[i], (off_t)size>> 1, SEEK_SET) < 0) {
+				if (errno != ENXIO)
+					pr_fail(stderr, "%s: lseek SEEK_SET on memfd failed: "
+						"errno=%d (%s)\n", name, errno, strerror(errno));
+			}
+#endif
+#if defined(SEEK_CUR)
+			if (lseek(fds[i], (off_t)0, SEEK_CUR) < 0) {
+				if (errno != ENXIO)
+					pr_fail(stderr, "%s: lseek SEEK_CUR on memfd failed: "
+						"errno=%d (%s)\n", name, errno, strerror(errno));
+			}
+#endif
+#if defined(SEEK_END)
+			if (lseek(fds[i], (off_t)0, SEEK_END) < 0) {
+				if (errno != ENXIO)
+					pr_fail(stderr, "%s: lseek SEEK_END on memfd failed: "
+						"errno=%d (%s)\n", name, errno, strerror(errno));
+			}
+#endif
+#if defined(SEEK_HOLE)
+			if (lseek(fds[i], (off_t)0, SEEK_HOLE) < 0) {
+				if (errno != ENXIO)
+					pr_fail(stderr, "%s: lseek SEEK_HOLE on memfd failed: "
+						"errno=%d (%s)\n", name, errno, strerror(errno));
+			}
+#endif
+#if defined(SEEK_DATA)
+			if (lseek(fds[i], (off_t)0, SEEK_DATA) < 0) {
+				if (errno != ENXIO)
+					pr_fail(stderr, "%s: lseek SEEK_DATA on memfd failed: "
+						"errno=%d (%s)\n", name, errno, strerror(errno));
+			}
+#endif
+			if (!opt_do_run)
+				goto clean;
+		}
+	
 clean:
 		for (i = 0; i < MAX_MEM_FDS; i++) {
 			if (maps[i] != MAP_FAILED)
@@ -151,7 +224,14 @@ int stress_memfd(
 	const char *name)
 {
 	pid_t pid;
-	uint32_t restarts = 0, nomems = 0;
+	uint32_t ooms = 0, segvs = 0, nomems = 0;
+
+	if (!set_memfd_bytes) {
+		if (opt_flags & OPT_FLAGS_MAXIMIZE)
+			opt_memfd_bytes = MAX_MEMFD_BYTES;
+		if (opt_flags & OPT_FLAGS_MINIMIZE)
+			opt_memfd_bytes = MIN_MEMFD_BYTES;
+	}
 
 again:
 	if (!opt_do_run)
@@ -182,13 +262,22 @@ again:
 				name, stress_strsignal(WTERMSIG(status)), instance);
 			/* If we got killed by OOM killer, re-start */
 			if ((WTERMSIG(status) == SIGKILL) ||
-			    (WTERMSIG(status) == SIGKILL)) {
+			    (WTERMSIG(status) == SIGSEGV)) {
+				log_system_mem_info();
 				pr_dbg(stderr, "%s: assuming killed by OOM killer, "
 					"restarting again (instance %d)\n",
 					name, instance);
-				restarts++;
+				ooms++;
 				goto again;
 			}
+
+		} else if (WTERMSIG(status) == SIGSEGV) {
+			pr_dbg(stderr, "%s: killed by SIGSEGV, "
+				"restarting again "
+				"(instance %d)\n",
+				name, instance);
+			segvs++;
+			goto again;
 		}
 	} else if (pid == 0) {
 		setpgid(0, pgrp);
@@ -198,10 +287,12 @@ again:
 
 		stress_memfd_allocs(name, counter, max_ops);
 	}
-	if (restarts + nomems > 0)
+
+	if (ooms + segvs + nomems > 0)
 		pr_dbg(stderr, "%s: OOM restarts: %" PRIu32
+			", SEGV restarts: %" PRIu32
 			", out of memory restarts: %" PRIu32 ".\n",
-			name, restarts, nomems);
+			name, ooms, segvs, nomems);
 
 	return EXIT_SUCCESS;
 }
