@@ -24,6 +24,10 @@
  */
 #define _GNU_SOURCE
 
+#include "stress-ng.h"
+
+#if defined(STRESS_MLOCK)
+
 #include <stddef.h>
 #include <stdio.h>
 #include <stdlib.h>
@@ -35,8 +39,6 @@
 #include <sys/wait.h>
 #include <signal.h>
 
-#include "stress-ng.h"
-
 #define MLOCK_MAX	(256*1024)
 
 #if defined(__NR_mlock2)
@@ -80,12 +82,12 @@ static int mlock_shim(const void *addr, size_t len)
 	}
 
 	/* Just do mlock */
-	return mlock(addr, len);
+	return mlock((void *)addr, len);
 }
 #else
 static inline int mlock_shim(const void *addr, size_t len)
 {
-	return mlock(addr, len);
+	return mlock((void *)addr, len);
 }
 #endif
 
@@ -103,8 +105,13 @@ int stress_mlock(
 	const size_t page_size = stress_get_pagesize();
 	pid_t pid;
 	size_t max = sysconf(_SC_MAPPED_FILES);
+	uint8_t **mappings;
 	max = max > MLOCK_MAX ? MLOCK_MAX : max;
 
+	if ((mappings = calloc(max, sizeof(uint8_t *))) == NULL) {
+                pr_fail_dbg(name, "malloc");
+                return EXIT_NO_RESOURCE;
+	}
 again:
 	pid = fork();
 	if (pid < 0) {
@@ -133,14 +140,22 @@ again:
 				instance);
 			/* If we got killed by OOM killer, re-start */
 			if (WTERMSIG(status) == SIGKILL) {
+				log_system_mem_info();
 				pr_dbg(stderr, "%s: assuming killed by OOM "
 					"killer, restarting again "
 					"(instance %d)\n", name, instance);
 				goto again;
 			}
+			/* If we got killed by sigsegv, re-start */
+			if (WTERMSIG(status) == SIGSEGV) {
+				pr_dbg(stderr, "%s: killed by SIGSEGV, "
+					"restarting again "
+					"(instance %d)\n",
+					name, instance);
+				goto again;
+			}
 		}
 	} else if (pid == 0) {
-		uint8_t *mappings[max];
 		size_t i, n;
 
 		setpgid(0, pgrp);
@@ -154,12 +169,12 @@ again:
 				if (!opt_do_run || (max_ops && *counter >= max_ops))
 					break;
 
-				mappings[n] = mmap(NULL, page_size * 3,
+				mappings[n] = (uint8_t *)mmap(NULL, page_size * 3,
 					PROT_READ | PROT_WRITE,
 					MAP_SHARED | MAP_ANONYMOUS, -1, 0);
 				if (mappings[n] == MAP_FAILED)
 					break;
-				ret = mlock_shim(mappings[n] + page_size, page_size);
+				ret = mlock_shim((void *)(mappings[n] + page_size), page_size);
 				if (ret < 0) {
 					if (errno == EAGAIN)
 						continue;
@@ -186,7 +201,7 @@ again:
 
 				addr ^= mlocked;
 				if (mlocked)
-					(void)munlock((uint8_t *)addr + page_size, page_size);
+					(void)munlock((void *)((uint8_t *)addr + page_size), page_size);
 				munmap((void *)addr, page_size * 3);
 			}
 #if !defined(__gnu_hurd__)
@@ -200,7 +215,7 @@ again:
 				if (!opt_do_run || (max_ops && *counter >= max_ops))
 					break;
 
-				mappings[n] = mmap(NULL, page_size,
+				mappings[n] = (uint8_t *)mmap(NULL, page_size,
 					PROT_READ | PROT_WRITE,
 					MAP_SHARED | MAP_ANONYMOUS, -1, 0);
 				if (mappings[n] == MAP_FAILED)
@@ -210,9 +225,13 @@ again:
 			(void)munlockall();
 #endif
 			for (i = 0; i < n;  i++)
-				munmap(mappings[i], page_size);
+				munmap((void *)mappings[i], page_size);
 		} while (opt_do_run && (!max_ops || *counter < max_ops));
 	}
 
+	free(mappings);
+
 	return EXIT_SUCCESS;
 }
+
+#endif
