@@ -49,9 +49,18 @@ int stress_mmapmany(
 {
 	const size_t page_size = stress_get_pagesize();
 	pid_t pid;
-	size_t max = sysconf(_SC_MAPPED_FILES);
+	ssize_t max = sysconf(_SC_MAPPED_FILES);
+	uint8_t **mappings;
 	max = STRESS_MAXIMUM(max, MMAP_MAX);
 
+	if (max < 1) {
+		pr_fail_dbg(name, "sysconf(_SC_MAPPED_FILES)");
+		return EXIT_NO_RESOURCE;
+	}
+	if ((mappings = calloc(max, sizeof(uint8_t *))) == NULL) {
+		pr_fail_dbg(name, "malloc");
+		return EXIT_NO_RESOURCE;
+	}
 again:
 	pid = fork();
 	if (pid < 0) {
@@ -84,10 +93,17 @@ again:
 					"(instance %d)\n", name, instance);
 				goto again;
 			}
+			/* If we got killed by sigsegv, re-start */
+			if (WTERMSIG(status) == SIGSEGV) {
+				pr_dbg(stderr, "%s: killed by SIGSEGV, "
+					"restarting again "
+					"(instance %d)\n",
+					name, instance);
+				goto again;
+			}
 		}
 	} else if (pid == 0) {
-		uint8_t *mappings[max];
-		size_t i, n;
+		ssize_t i, n;
 
 		setpgid(0, pgrp);
 		stress_parent_died_alarm();
@@ -100,23 +116,26 @@ again:
 				if (!opt_do_run || (max_ops && *counter >= max_ops))
 					break;
 
-				mappings[n] = mmap(NULL, page_size * 3,
+				mappings[n] = (uint8_t *)mmap(NULL,
+					page_size * 3,
 					PROT_READ | PROT_WRITE,
 					MAP_PRIVATE | MAP_ANONYMOUS, -1, 0);
 				if (mappings[n] == MAP_FAILED)
 					break;
-				if (munmap(mappings[n] + page_size, page_size) < 0)
+				if (munmap((void *)(mappings[n] + page_size), page_size) < 0)
 					break;
 				(*counter)++;
 			}
 
 			for (i = 0; i < n;  i++) {
-				munmap(mappings[i], page_size);
-				munmap(mappings[i] + page_size, page_size);
-				munmap(mappings[i] + page_size + page_size, page_size);
+				munmap((void *)mappings[i], page_size);
+				munmap((void *)(mappings[i] + page_size), page_size);
+				munmap((void *)(mappings[i] + page_size + page_size), page_size);
 			}
 		} while (opt_do_run && (!max_ops || *counter < max_ops));
 	}
 
+	free(mappings);
+
 	return EXIT_SUCCESS;
 }
