@@ -34,7 +34,9 @@
 #include <unistd.h>
 #include <inttypes.h>
 #include <semaphore.h>
+#if defined(HAVE_LIB_PTHREAD)
 #include <pthread.h>
+#endif
 #include <signal.h>
 #include <sys/time.h>
 #include <sys/times.h>
@@ -82,7 +84,8 @@ typedef unsigned long int __kernel_ulong_t;
 #define STRESS_CONCAT(a, b) a ## b
 #define STRESS_CONCAT_EXPAND(a, b) STRESS_CONCAT(a, b)
 #define STRESS_ASSERT(expr) \
-	enum { STRESS_CONCAT_EXPAND(STRESS_ASSERT_AT_LINE_, __LINE__) = 1 / !!(expr) };
+	enum { STRESS_CONCAT_EXPAND(STRESS_ASSERT_AT_LINE_, __LINE__) = \
+		1 / !!(expr) };
 
 #define STRESS_MINIMUM(a,b) (((a) < (b)) ? (a) : (b))
 #define STRESS_MAXIMUM(a,b) (((a) > (b)) ? (a) : (b))
@@ -193,6 +196,13 @@ typedef unsigned long int __kernel_ulong_t;
 	 OPT_FLAGS_STACK_FILL | OPT_FLAGS_CACHE_PREFETCH | \
 	 OPT_FLAGS_AGGRESSIVE | OPT_FLAGS_IGNITE_CPU)
 
+#define WARN_ONCE_NO_CACHE	0x00000001	/* No /sys/../cpu0/cache */
+#define WARN_ONCE_CACHE_DEFAULT	0x00000002	/* default cache size */
+#define WARN_ONCE_CACHE_NONE	0x00000004	/* no cache info */
+#define WARN_ONCE_CACHE_WAY	0x00000008	/* cache way too high */
+#define WARN_ONCE_CACHE_SIZE	0x00000010	/* cache size info */
+#define WARN_ONCE_CACHE_REDUCED	0x00000020	/* reduced cache */
+
 
 /* Stressor classes */
 #define CLASS_CPU		0x00000001	/* CPU only */
@@ -373,6 +383,15 @@ extern void pr_openlog(const char *filename);
 #define MAX_MATRIX_SIZE		(4096)
 #define DEFAULT_MATRIX_SIZE	(256)
 
+#define MIN_MEMFD_BYTES		(2 * MB)
+#if UINTPTR_MAX == MAX_32
+#define MAX_MEMFD_BYTES		(MAX_32)
+#else
+#define MAX_MEMFD_BYTES		(4 * GB)
+#endif
+#define DEFAULT_MEMFD_BYTES	(256 * MB)
+
+
 #define MIN_MERGESORT_SIZE	(1 * KB)
 #define MAX_MERGESORT_SIZE	(4 * MB)
 #define DEFAULT_MERGESORT_SIZE	(256 * KB)
@@ -393,6 +412,14 @@ extern void pr_openlog(const char *filename);
 #endif
 #define DEFAULT_MREMAP_BYTES	(256 * MB)
 
+#define MIN_MSYNC_BYTES		(1 * MB)	/* MUST NOT BE page size or less! */
+#if UINTPTR_MAX == MAX_32
+#define MAX_MSYNC_BYTES		(MAX_32)
+#else
+#define MAX_MSYNC_BYTES		(4 * GB)
+#endif
+#define DEFAULT_MSYNC_BYTES	(256 * MB)
+
 #define MIN_PTHREAD		(1)
 #define MAX_PTHREAD		(30000)
 #define DEFAULT_PTHREAD		(1024)
@@ -405,6 +432,10 @@ extern void pr_openlog(const char *filename);
 #define MAX_READAHEAD_BYTES	(256ULL * GB)
 #define DEFAULT_READAHEAD_BYTES	(1 * GB)
 
+#define MIN_SCTP_PORT		(1024)
+#define MAX_SCTP_PORT		(65535)
+#define DEFAULT_SCTP_PORT	(9000)
+
 #define MIN_SENDFILE_SIZE	(1 * KB)
 #define MAX_SENDFILE_SIZE	(1 * GB)
 #define DEFAULT_SENDFILE_SIZE	(4 * MB)
@@ -543,9 +574,12 @@ extern void pr_openlog(const char *filename);
 #define STRESS_CPU_DITHER_Y	(768)
 
 #define STRESS_NBITS(a)		(sizeof(a[0]) * 8)
-#define STRESS_GETBIT(a, i)	(a[i / STRESS_NBITS(a)] & (1 << (i & (STRESS_NBITS(a)-1))))
-#define STRESS_CLRBIT(a, i)	(a[i / STRESS_NBITS(a)] &= ~(1 << (i & (STRESS_NBITS(a)-1))))
-#define STRESS_SETBIT(a, i)	(a[i / STRESS_NBITS(a)] |= (1 << (i & (STRESS_NBITS(a)-1))))
+#define STRESS_GETBIT(a, i)	(a[i / STRESS_NBITS(a)] & \
+				 (1 << (i & (STRESS_NBITS(a)-1))))
+#define STRESS_CLRBIT(a, i)	(a[i / STRESS_NBITS(a)] &= \
+				 ~(1 << (i & (STRESS_NBITS(a)-1))))
+#define STRESS_SETBIT(a, i)	(a[i / STRESS_NBITS(a)] |= \
+				 (1 << (i & (STRESS_NBITS(a)-1))))
 
 #define SIEVE_SIZE 		(10000000)
 
@@ -556,10 +590,22 @@ extern void pr_openlog(const char *filename);
 
 #define SIZEOF_ARRAY(a)		(sizeof(a) / sizeof(a[0]))
 
-#if defined(__x86_64__) || defined(__x86_64) || defined(__i386__) || defined(__i386)
+#if defined(__x86_64__) || defined(__x86_64) || \
+    defined(__i386__) || defined(__i386)
 #define STRESS_X86	1
 #endif
 
+#if defined(__ARM_ARCH_6__) || defined(__ARM_ARCH_6J__) ||     \
+    defined(__ARM_ARCH_6K__) || defined(__ARM_ARCH_6Z__) ||    \
+    defined(__ARM_ARCH_6ZK__) || defined(__ARM_ARCH_6T2__) ||  \
+    defined(__ARM_ARCH_6M__) ||  defined(__ARM_ARCH_7__) ||    \
+    defined(__ARM_ARCH_7A__) || defined(__ARM_ARCH_7R__) ||    \
+    defined(__ARM_ARCH_7M__) || defined(__ARM_ARCH_7EM__) ||   \
+    defined(__ARM_ARCH_8A__) || defined(__aarch64__)
+#define STRESS_ARM      1
+#endif
+
+/* gcc 4.7 and later support vector ops */
 #if NEED_GNUC(4,7,0)
 #define STRESS_VECTOR	1
 #endif
@@ -575,6 +621,7 @@ extern void pr_openlog(const char *filename);
 #define RESTRICT
 #endif
 
+/*Specific compilers have __uint128_t type support */
 #if defined(__GNUC__) && !defined(__clang__) && defined(__SIZEOF_INT128__)
 #define STRESS_INT128	1
 #endif
@@ -588,13 +635,28 @@ extern void pr_openlog(const char *filename);
 #endif
 
 #if defined(__linux__)
-#define STRESS_IOPRIO
+/*
+ *  See ioprio_set(2) and linux/ioprio.h, glibc has no definitions
+ *  for these at present. Also refer to Documentation/block/ioprio.txt
+ *  in the Linux kernel source.
+ */
+#define IOPRIO_CLASS_RT         (1)
+#define IOPRIO_CLASS_BE         (2)
+#define IOPRIO_CLASS_IDLE       (3)
+
+#define IOPRIO_WHO_PROCESS      (1)
+#define IOPRIO_WHO_PGRP         (2)
+#define IOPRIO_WHO_USER         (3)
+
+#define IOPRIO_PRIO_VALUE(class, data)  (((class) << 13) | data)
 #endif
 
+/* malloc mallopt() allocation parameter support */
 #if defined(__GNUC__) && defined(__linux__)
 #define STRESS_MALLOPT
 #endif
 
+/* -O3 attribute support */
 #if defined(__GNUC__) && !defined(__clang__) && NEED_GNUC(4,6,0)
 #define OPTIMIZE3 __attribute__((optimize("-O3")))
 #else
@@ -611,6 +673,7 @@ extern void pr_openlog(const char *filename);
 #define FORCE_DO_NOTHING() while (0)
 #endif
 
+/* prctl(2) timer slack support */
 #if defined(__linux__) && \
     defined(PR_SET_TIMERSLACK) && \
     defined(PR_GET_TIMERSLACK)
@@ -647,7 +710,7 @@ typedef struct {
 #define HOT
 #endif
 
-#if defined(__GNUC__) && NEED_GNUC(4,6,0)
+#if defined(__GNUC__) && NEED_GNUC(4,6,0) && !defined(__sun__)
 #define MLOCKED __attribute__((__section__("mlocked")))
 #define MLOCKED_SECTION 1
 #else
@@ -761,7 +824,20 @@ typedef struct {
 	uint64_t mem_cache_size;			/* Bytes */
 	uint16_t mem_cache_level;			/* 1=L1, 2=L2, 3=L3 */
 	uint32_t mem_cache_ways;			/* cache ways size */
+	struct {
+		uint32_t	flags;			/* flag bits */
+#if defined(HAVE_LIB_PTHREAD)
+		pthread_spinlock_t lock;		/* protection lock */
+#endif
+	} warn_once;
+	uint32_t warn_once_flags;			/* Warn once flags */
 	uint8_t  str_shared[STR_SHARED_SIZE];		/* str copying buffer */
+	struct {
+		uint64_t val64;
+		uint32_t val32;
+		uint16_t val16;
+		uint8_t	 val8;
+	} atomic;					/* Shared atomic temp vars */
 	struct {
 		uint32_t futex[STRESS_PROCS_MAX];	/* Shared futexes */
 		uint64_t timeout[STRESS_PROCS_MAX];	/* Shared futex timeouts */
@@ -822,6 +898,10 @@ typedef enum {
 #if defined(__linux__) && defined(HAVE_APPARMOR)
 	__STRESS_APPARMOR,
 #define STRESS_APPARMOR __STRESS_APPARMOR
+#endif
+#if defined(HAVE_ATOMIC)
+	__STRESS_ATOMIC,
+#define STRESS_ATOMIC __STRESS_ATOMIC
 #endif
 	STRESS_BRK,
 	STRESS_BSEARCH,
@@ -837,6 +917,7 @@ typedef enum {
 #endif
 	STRESS_CHDIR,
 	STRESS_CHMOD,
+	STRESS_CHOWN,
 #if defined(HAVE_LIB_RT) && _POSIX_C_SOURCE >= 199309L
 	__STRESS_CLOCK,
 #define STRESS_CLOCK __STRESS_CLOCK
@@ -894,6 +975,10 @@ typedef enum {
 	STRESS_FORK,
 	STRESS_FP_ERROR,
 	STRESS_FSTAT,
+#if defined(__linux__)
+	__STRESS_FULL,
+#define STRESS_FULL __STRESS_FULL
+#endif
 #if defined(__linux__) && defined(__NR_futex)
 	__STRESS_FUTEX,
 #define STRESS_FUTEX __STRESS_FUTEX
@@ -918,13 +1003,18 @@ typedef enum {
 #define STRESS_HEAPSORT __STRESS_HEAPSORT
 #endif
 	STRESS_HSEARCH,
-#if defined(STRESS_X86) && defined(__GNUC__) && NEED_GNUC(4,6,0)
+#if (defined(STRESS_X86) || defined(STRESS_ARM)) && \
+    defined(__GNUC__) && NEED_GNUC(4,6,0)
 	__STRESS_ICACHE,
 #define STRESS_ICACHE __STRESS_ICACHE
 #endif
 #if defined(__linux__) && NEED_GLIBC(2,9,0)
 	__STRESS_INOTIFY,
 #define STRESS_INOTIFY __STRESS_INOTIFY
+#endif
+#if defined(__linux__) && defined(__NR_ioprio_set) && defined(__NR_ioprio_get)
+	__STRESS_IOPRIO,
+#define STRESS_IOPRIO __STRESS_IOPRIO
 #endif
 	STRESS_IOSYNC,
 	STRESS_ITIMER,
@@ -946,17 +1036,32 @@ typedef enum {
 #define STRESS_LEASE __STRESS_LEASE
 #endif
 	STRESS_LINK,
-#if (defined(__GNUC__) || defined(__clang__)) && defined(STRESS_X86) && defined(__linux__)
+#if (((defined(__GNUC__) || defined(__clang__)) && defined(STRESS_X86)) || \
+    (defined(__GNUC__) && NEED_GNUC(4,7,0) && defined(STRESS_ARM))) && defined(__linux__)
 	__STRESS_LOCKBUS,
 #define STRESS_LOCKBUS __STRESS_LOCKBUS
 #endif
+#if defined(F_GETLK) && defined(F_SETLK) && defined(F_SETLKW) && \
+    defined(F_WRLCK) && defined(F_UNLCK)
+	__STRESS_LOCKA,
+#define STRESS_LOCKA __STRESS_LOCKA
+#endif
 #if _BSD_SOURCE || _SVID_SOURCE || _XOPEN_SOURCE >= 500 || \
      (_XOPEN_SOURCE && _XOPEN_SOURCE_EXTENDED)
 	__STRESS_LOCKF,
 #define STRESS_LOCKF __STRESS_LOCKF
+#endif
+#if defined(F_OFD_GETLK) && defined(F_OFD_SETLK) && defined(F_OFD_SETLKW) && \
+    defined(F_WRLCK) && defined(F_UNLCK)
+	__STRESS_LOCKOFD,
+#define STRESS_LOCKOFD __STRESS_LOCKOFD
 #endif
 	STRESS_LONGJMP,
 	STRESS_LSEARCH,
+#if !defined(__gnu_hurd__) && NEED_GLIBC(2,19,0)
+	__STRESS_MADVISE,
+#define STRESS_MADVISE __STRESS_MADVISE
+#endif
 	STRESS_MALLOC,
 	STRESS_MATRIX,
 #if defined(HAVE_LIB_PTHREAD) && defined(__linux__) && defined(__NR_membarrier)
@@ -977,7 +1082,7 @@ typedef enum {
 #define STRESS_MINCORE __STRESS_MINCORE
 #endif
 	STRESS_MKNOD,
-#if defined(_POSIX_MEMLOCK_RANGE)
+#if defined(_POSIX_MEMLOCK_RANGE) && !defined(__minix__)
 	__STRESS_MLOCK,
 #define STRESS_MLOCK __STRESS_MLOCK
 #endif
@@ -995,6 +1100,10 @@ typedef enum {
 	__STRESS_MSG,
 #define STRESS_MSG __STRESS_MSG
 #endif
+#if !defined(__gnu_hurd__) && !defined(__minix__)
+	__STRESS_MSYNC,
+#define STRESS_MSYNC __STRESS_MSYNC
+#endif
 #if defined(HAVE_LIB_RT) && defined(__linux__)
 	__STRESS_MQ,
 #define STRESS_MQ __STRESS_MQ
@@ -1013,6 +1122,10 @@ typedef enum {
 #if defined(__linux__) && defined(F_SETPIPE_SZ)
 	__STRESS_OOM_PIPE,
 #define STRESS_OOM_PIPE __STRESS_OOM_PIPE
+#endif
+#if defined(__linux__)
+	__STRESS_OPCODE,
+#define STRESS_OPCODE __STRESS_OPCODE
 #endif
 	STRESS_OPEN,
 #if defined(__linux__)
@@ -1032,6 +1145,10 @@ typedef enum {
 #if defined(__linux__)
 	__STRESS_PTRACE,
 #define STRESS_PTRACE __STRESS_PTRACE
+#endif
+#if defined(__linux__)
+	__STRESS_PTY,
+#define STRESS_PTY __STRESS_PTY
 #endif
 	STRESS_QSORT,
 #if defined(__linux__) && (		\
@@ -1060,6 +1177,18 @@ typedef enum {
 	__STRESS_RLIMIT,
 #define STRESS_RLIMIT __STRESS_RLIMIT
 #endif
+#if defined(__linux__)
+	__STRESS_RTC,
+#define STRESS_RTC __STRESS_RTC
+#endif
+#if defined(HAVE_LIB_SCTP)
+	__STRESS_SCTP,
+#define STRESS_SCTP __STRESS_SCTP
+#endif
+#if defined(__linux__) && defined(__NR_memfd_create)
+	__STRESS_SEAL,
+#define STRESS_SEAL __STRESS_SEAL
+#endif
 #if defined(HAVE_SECCOMP_H) && defined(__linux__) && defined(PR_SET_SECCOMP)
 	__STRESS_SECCOMP,
 #define STRESS_SECCOMP __STRESS_SECCOMP
@@ -1104,13 +1233,19 @@ typedef enum {
 #define STRESS_SOCKET_FD __STRESS_SOCKET_FD
 #endif
 	STRESS_SOCKET_PAIR,
+#if defined(__linux__)
 	__STRESS_SPAWN,
 #define STRESS_SPAWN __STRESS_SPAWN
+#endif
 #if defined(__linux__) && NEED_GLIBC(2,5,0)
 	__STRESS_SPLICE,
 #define STRESS_SPLICE __STRESS_SPLICE
 #endif
 	STRESS_STACK,
+#if defined(__linux__)
+	__STRESS_STACKMMAP,
+#define STRESS_STACKMMAP __STRESS_STACKMMAP
+#endif
 	STRESS_STR,
 	STRESS_STREAM,
 	STRESS_SWITCH,
@@ -1136,6 +1271,10 @@ typedef enum {
 	__STRESS_TIMERFD,
 #define STRESS_TIMERFD __STRESS_TIMERFD
 #endif
+#if defined(__linux__)
+	__STRESS_TLB_SHOOTDOWN,
+#define STRESS_TLB_SHOOTDOWN __STRESS_TLB_SHOOTDOWN
+#endif
 #if defined(STRESS_X86) && !defined(__OpenBSD__) && NEED_GNUC(4,6,0)
 	__STRESS_TSC,
 #define STRESS_TSC __STRESS_TSC
@@ -1160,7 +1299,7 @@ typedef enum {
 #define STRESS_USERFAULTFD __STRESS_USERFAULTFD
 #endif
 	STRESS_UTIME,
-#if defined(STRESS_VECTOR)
+#if defined(HAVE_VECMATH)
 	__STRESS_VECMATH,
 #define STRESS_VECMATH __STRESS_VECMATH
 #endif
@@ -1186,7 +1325,7 @@ typedef enum {
 	__STRESS_XATTR,
 #define STRESS_XATTR __STRESS_XATTR
 #endif
-#if defined(_POSIX_PRIORITY_SCHEDULING)
+#if defined(_POSIX_PRIORITY_SCHEDULING) && !defined(__minix__)
 	__STRESS_YIELD,
 #define STRESS_YIELD __STRESS_YIELD
 #endif
@@ -1277,6 +1416,11 @@ typedef enum {
 	OPT_APPARMOR_OPS,
 #endif
 
+#if defined(STRESS_ATOMIC)
+	OPT_ATOMIC,
+	OPT_ATOMIC_OPS,
+#endif
+
 	OPT_BRK,
 	OPT_BRK_OPS,
 	OPT_BRK_NOTOUCH,
@@ -1313,6 +1457,9 @@ typedef enum {
 	OPT_CHMOD,
 	OPT_CHMOD_OPS,
 
+	OPT_CHOWN,
+	OPT_CHOWN_OPS,
+
 #if defined(STRESS_CLOCK)
 	OPT_CLOCK,
 	OPT_CLOCK_OPS,
@@ -1417,6 +1564,11 @@ typedef enum {
 	OPT_FSTAT_OPS,
 	OPT_FSTAT_DIR,
 
+#if defined(STRESS_FULL)
+	OPT_FULL,
+	OPT_FULL_OPS,
+#endif
+
 	OPT_FUTEX,
 	OPT_FUTEX_OPS,
 
@@ -1468,6 +1620,11 @@ typedef enum {
 	OPT_IONICE_LEVEL,
 #endif
 
+#if defined(STRESS_IOPRIO)
+	OPT_IOPRIO,
+	OPT_IOPRIO_OPS,
+#endif
+
 	OPT_IOSYNC_OPS,
 
 	OPT_ITIMER,
@@ -1506,12 +1663,22 @@ typedef enum {
 	OPT_LOCKBUS_OPS,
 #endif
 
+#if defined(STRESS_LOCKA)
+	OPT_LOCKA,
+	OPT_LOCKA_OPS,
+#endif
+
 #if defined(STRESS_LOCKF)
 	OPT_LOCKF,
 	OPT_LOCKF_OPS,
 	OPT_LOCKF_NONBLOCK,
 #endif
 
+#if defined(STRESS_LOCKOFD)
+	OPT_LOCKOFD,
+	OPT_LOCKOFD_OPS,
+#endif
+
 	OPT_LOG_BRIEF,
 	OPT_LOG_FILE,
 
@@ -1522,6 +1689,11 @@ typedef enum {
 	OPT_LSEARCH_OPS,
 	OPT_LSEARCH_SIZE,
 
+#if defined(STRESS_MADVISE)
+	OPT_MADVISE,
+	OPT_MADVISE_OPS,
+#endif
+
 	OPT_MALLOC,
 	OPT_MALLOC_OPS,
 	OPT_MALLOC_BYTES,
@@ -1548,6 +1720,7 @@ typedef enum {
 #if defined(STRESS_MEMFD)
 	OPT_MEMFD,
 	OPT_MEMFD_OPS,
+	OPT_MEMFD_BYTES,
 #endif
 
 #if defined(STRESS_MERGESORT)
@@ -1598,6 +1771,12 @@ typedef enum {
 	OPT_MSG,
 	OPT_MSG_OPS,
 
+#if defined(STRESS_MSYNC)
+	OPT_MSYNC,
+	OPT_MSYNC_BYTES,
+	OPT_MSYNC_OPS,
+#endif
+
 #if defined(STRESS_MQ)
 	OPT_MQ,
 	OPT_MQ_OPS,
@@ -1622,6 +1801,11 @@ typedef enum {
 	OPT_OOM_PIPE_OPS,
 #endif
 
+#if defined(STRESS_OPCODE)
+	OPT_OPCODE,
+	OPT_OPCODE_OPS,
+#endif
+
 	OPT_OPEN_OPS,
 
 #if defined(STRESS_PAGE_IN)
@@ -1660,6 +1844,11 @@ typedef enum {
 	OPT_PTRACE,
 	OPT_PTRACE_OPS,
 
+#if defined(STRESS_PTY)
+	OPT_PTY,
+	OPT_PTY_OPS,
+#endif
+
 	OPT_QSORT,
 	OPT_QSORT_OPS,
 	OPT_QSORT_INTEGERS,
@@ -1690,9 +1879,26 @@ typedef enum {
 	OPT_RLIMIT_OPS,
 #endif
 
+#if defined(STRESS_RTC)
+	OPT_RTC,
+	OPT_RTC_OPS,
+#endif
+
 	OPT_SCHED,
 	OPT_SCHED_PRIO,
 
+#if defined(STRESS_SCTP)
+	OPT_SCTP,
+	OPT_SCTP_OPS,
+	OPT_SCTP_DOMAIN,
+	OPT_SCTP_PORT,
+#endif
+
+#if defined(STRESS_SEAL)
+	OPT_SEAL,
+	OPT_SEAL_OPS,
+#endif
+
 #if defined(STRESS_SECCOMP)
 	OPT_SECCOMP,
 	OPT_SECCOMP_OPS,
@@ -1770,6 +1976,7 @@ typedef enum {
 	OPT_SOCKET_NODELAY,
 	OPT_SOCKET_OPTS,
 	OPT_SOCKET_PORT,
+	OPT_SOCKET_TYPE,
 
 #if defined(STRESS_SOCKET_FD)
 	OPT_SOCKET_FD,
@@ -1794,6 +2001,11 @@ typedef enum {
 	OPT_STACK_OPS,
 	OPT_STACK_FILL,
 
+#if defined(STRESS_STACKMMAP)
+	OPT_STACKMMAP,
+	OPT_STACKMMAP_OPS,
+#endif
+
 	OPT_STR,
 	OPT_STR_OPS,
 	OPT_STR_METHOD,
@@ -1826,6 +2038,8 @@ typedef enum {
 	OPT_TEE_OPS,
 #endif
 
+	OPT_TASKSET,
+
 	OPT_TEMP_PATH,
 
 	OPT_THERMAL_ZONES,
@@ -1848,6 +2062,11 @@ typedef enum {
 #endif
 	OPT_TIMES,
 
+#if defined(STRESS_TLB_SHOOTDOWN)
+	OPT_TLB_SHOOTDOWN,
+	OPT_TLB_SHOOTDOWN_OPS,
+#endif
+
 #if defined(STRESS_TSC)
 	OPT_TSC,
 	OPT_TSC_OPS,
@@ -2027,6 +2246,11 @@ extern volatile bool opt_sigint;	/* true if stopped by SIGINT */
 extern mwc_t __mwc;			/* internal mwc random state */
 extern pid_t pgrp;			/* proceess group leader */
 
+/* syscall shims not provided by glibc */
+extern int sys_ioprio_set(int which, int who, int ioprio);
+extern int sys_ioprio_get(int which, int who);
+
+
 /*
  *  externs to force gcc to stash computed values and hence
  *  to stop the optimiser optimising code away to zero. The
@@ -2048,12 +2272,15 @@ static inline void double_put(const double a)
 }
 
 /* Filenames and directories */
-extern int stress_temp_filename(char *path, const size_t len, const char *name,
-	const pid_t pid, const uint32_t instance, const uint64_t magic);
-extern int stress_temp_dir(char *path, const size_t len, const char *name,
-	const pid_t pid, const uint32_t instance);
-extern WARN_UNUSED int stress_temp_dir_mk(const char *name, const pid_t pid, const uint32_t instance);
-extern int stress_temp_dir_rm(const char *name, const pid_t pid, const uint32_t instance);
+extern int stress_temp_filename(char *path, const size_t len,
+	const char *name, const pid_t pid, const uint32_t instance,
+	const uint64_t magic);
+extern int stress_temp_dir(char *path, const size_t len,
+	const char *name, const pid_t pid, const uint32_t instance);
+extern WARN_UNUSED int stress_temp_dir_mk(const char *name, const pid_t pid,
+	const uint32_t instance);
+extern int stress_temp_dir_rm(const char *name, const pid_t pid,
+	const uint32_t instance);
 extern void stress_cwd_readwriteable(void);
 
 extern const char *stress_strsignal(const int signum);
@@ -2065,18 +2292,25 @@ static inline void clflush(volatile void *ptr)
         asm volatile("clflush %0" : "+m" (*(volatile char *)ptr));
 }
 
-static inline void mfence(void)
-{
-	asm volatile("mfence" : : : "memory");
-}
-
 #else
 
 #define clflush(ptr)	do { } while (0) /* No-op */
-#define mfence()	do { } while (0) /* No-op */
 
 #endif
 
+static inline void mfence(void)
+{
+#if NEED_GNUC(4, 2, 0) 
+	__sync_synchronize();
+#else
+#if defined(STRESS_X86)
+	asm volatile("mfence" : : : "memory");
+#else
+	/* Other arches not yet implemented for older GCC flavours */
+#endif
+#endif
+}
+
 /*
  *  mwc_seed()
  *      set mwc seeds
@@ -2125,7 +2359,7 @@ extern void mwc_reseed(void);
  *  timeval_to_double()
  *      convert timeval to seconds as a double
  */
-static inline double timeval_to_double(const struct timeval *tv)
+static inline WARN_UNUSED double timeval_to_double(const struct timeval *tv)
 {
         return (double)tv->tv_sec + ((double)tv->tv_usec / 1000000.0);
 }
@@ -2161,26 +2395,28 @@ extern void set_proc_name(const char *name);
 extern int stress_mlock_region(const void *addr_start, const void *addr_end);
 
 /* Argument parsing and range checking */
-extern WARN_UNUSED int32_t get_opt_sched(const char *const str);
-extern WARN_UNUSED int32_t get_opt_ionice_class(const char *const str);
-extern WARN_UNUSED unsigned long get_unsigned_long(const char *const str);
-extern WARN_UNUSED int32_t get_int32(const char *const str);
+extern WARN_UNUSED int32_t  get_opt_sched(const char *const str);
+extern WARN_UNUSED int32_t  get_opt_ionice_class(const char *const str);
+extern WARN_UNUSED uint32_t get_uint32(const char *const str);
+extern WARN_UNUSED int32_t  get_int32(const char *const str);
 extern WARN_UNUSED uint64_t get_uint64(const char *const str);
-extern WARN_UNUSED uint64_t get_uint64_scale(const char *const str, const scale_t scales[],
-	const char *const msg);
+extern WARN_UNUSED uint64_t get_uint64_scale(const char *const str,
+	const scale_t scales[], const char *const msg);
 extern WARN_UNUSED uint64_t get_uint64_byte(const char *const str);
 extern WARN_UNUSED uint64_t get_uint64_time(const char *const str);
 extern void check_value(const char *const msg, const int val);
 extern void check_range(const char *const opt, const uint64_t val,
 	const uint64_t lo, const uint64_t hi);
+extern WARN_UNUSED int set_cpu_affinity(char *const arg);
 
 /* Misc helper funcs */
-extern char *munge_underscore(char *str);
+extern void log_system_mem_info(void);
+extern WARN_UNUSED char *munge_underscore(char *str);
 extern size_t stress_get_pagesize(void);
-extern int32_t stress_get_processors_online(void);
-extern int32_t stress_get_processors_configured(void);
-extern int32_t stress_get_ticks_per_second(void);
-extern ssize_t stress_get_stack_direction(const void *val1);
+extern WARN_UNUSED int32_t stress_get_processors_online(void);
+extern WARN_UNUSED int32_t stress_get_processors_configured(void);
+extern WARN_UNUSED int32_t stress_get_ticks_per_second(void);
+extern WARN_UNUSED ssize_t stress_get_stack_direction(const void *val1);
 extern void stress_get_memlimits(size_t *shmall, size_t *freemem, size_t *totalmem);
 extern WARN_UNUSED int stress_get_load_avg(double *min1, double *min5, double *min15);
 extern void set_max_limits(void);
@@ -2196,6 +2432,7 @@ extern void stress_cache_free(void);
 extern void ignite_cpu_start(void);
 extern void ignite_cpu_stop(void);
 extern int system_write(const char *path, const char *buf, const size_t buf_len);
+extern WARN_UNUSED int stress_set_nonblock(const int fd);
 extern WARN_UNUSED int system_read(const char *path, char *buf, const size_t buf_len);
 extern WARN_UNUSED uint64_t stress_get_prime64(const uint64_t n);
 extern WARN_UNUSED size_t stress_get_file_limit(void);
@@ -2213,6 +2450,34 @@ static inline int exit_status(const int err)
 		EXIT_NO_RESOURCE : EXIT_FAILURE;
 }
 
+/*
+ *  Stack aligning for clone() system calls
+ *	align to nearest 16 bytes for aarch64 et al,
+ *	assumes we have enough slop to do this
+ */
+static inline WARN_UNUSED void *align_stack(void *stack_top)
+{
+	return (void *)((uintptr_t)stack_top & ~(uintptr_t)0xf);
+
+}
+
+/*
+ *  Check if flag is set, and set flag
+ */
+static inline WARN_UNUSED uint32_t warn_once(const uint32_t flag)
+{
+	uint32_t tmp;
+
+#if defined(HAVE_LIB_PTHREAD)
+	pthread_spin_lock(&shared->warn_once.lock);
+#endif
+	tmp = !(shared->warn_once.flags & flag);
+	shared->warn_once.flags |= flag;
+#if defined(HAVE_LIB_PTHREAD)
+	pthread_spin_unlock(&shared->warn_once.lock);
+#endif
+	return tmp;
+}
 
 /* Memory tweaking */
 extern int madvise_random(void *addr, const size_t length);
@@ -2232,12 +2497,16 @@ extern void tz_dump(FILE *fp, const shared_t *shared, const stress_t stressors[]
 #endif
 
 /* Network helpers */
+
+#define NET_ADDR_ANY		(0)
+#define NET_ADDR_LOOPBACK	(1)
+
 extern void stress_set_net_port(const char *optname, const char *optarg,
 	const int min_port, const int max_port, int *port);
 extern WARN_UNUSED int stress_set_net_domain(const int domain_mask, const char *name, const char *domain_name, int *domain);
 extern void stress_set_sockaddr(const char *name, const uint32_t instance,
 	const pid_t ppid, const int domain, const int port,
-	struct sockaddr **sockaddr, socklen_t *len);
+	struct sockaddr **sockaddr, socklen_t *len, int net_addr);
 extern void stress_set_sockaddr_port(const int domain, const int port, struct sockaddr *sockaddr);
 
 extern void stress_semaphore_posix_init(void);
@@ -2274,7 +2543,7 @@ extern int  stress_set_epoll_domain(const char *optarg);
 extern void stress_set_exec_max(const char *optarg);
 extern void stress_set_fallocate_bytes(const char *optarg);
 extern void stress_set_fifo_readers(const char *optarg);
-extern int stress_filename_opts(const char *opt);
+extern int  stress_filename_opts(const char *opt);
 extern void stress_set_fork_max(const char *optarg);
 extern void stress_set_fstat_dir(const char *optarg);
 extern void stress_set_hdd_bytes(const char *optarg);
@@ -2290,16 +2559,20 @@ extern void stress_set_malloc_max(const char *optarg);
 extern void stress_set_malloc_threshold(const char *optarg);
 extern int  stress_set_matrix_method(const char *name);
 extern void stress_set_matrix_size(const char *optarg);
+extern void stress_set_memfd_bytes(const char *optarg);
 extern void stress_set_mergesort_size(const void *optarg);
 extern void stress_set_mmap_bytes(const char *optarg);
-extern void stress_set_mremap_bytes(const char *optarg);
 extern void stress_set_mq_size(const char *optarg);
+extern void stress_set_mremap_bytes(const char *optarg);
+extern void stress_set_msync_bytes(const char *optarg);
 extern void stress_set_pipe_data_size(const char *optarg);
 extern void stress_set_pipe_size(const char *optarg);
 extern void stress_set_pthread_max(const char *optarg);
 extern void stress_set_qsort_size(const void *optarg);
 extern int  stress_rdrand_supported(void);
 extern void stress_set_readahead_bytes(const char *optarg);
+extern int  stress_set_sctp_domain(const char *optarg);
+extern void stress_set_sctp_port(const char *optarg);
 extern void stress_set_seek_size(const char *optarg);
 extern void stress_set_sendfile_size(const char *optarg);
 extern void stress_set_semaphore_posix_procs(const char *optarg);
@@ -2311,6 +2584,7 @@ extern void stress_set_shm_sysv_segments(const char *optarg);
 extern void stress_set_sleep_max(const char *optarg);
 extern int  stress_set_socket_domain(const char *name);
 extern int  stress_set_socket_opts(const char *optarg);
+extern int  stress_set_socket_type(const char *optarg);
 extern void stress_set_socket_port(const char *optarg);
 extern void stress_set_splice_bytes(const char *optarg);
 extern int  stress_set_str_method(const char *name);
@@ -2342,8 +2616,9 @@ extern int name(uint64_t *const counter, const uint32_t instance,	\
 STRESS(stress_affinity);
 STRESS(stress_af_alg);
 STRESS(stress_aio);
-STRESS(stress_aio_linux);
+STRESS(stress_aiol);
 STRESS(stress_apparmor);
+STRESS(stress_atomic);
 STRESS(stress_bigheap);
 STRESS(stress_bind_mount);
 STRESS(stress_brk);
@@ -2352,6 +2627,7 @@ STRESS(stress_cache);
 STRESS(stress_cap);
 STRESS(stress_chdir);
 STRESS(stress_chmod);
+STRESS(stress_chown);
 STRESS(stress_clock);
 STRESS(stress_clone);
 STRESS(stress_context);
@@ -2376,6 +2652,7 @@ STRESS(stress_flock);
 STRESS(stress_fork);
 STRESS(stress_fp_error);
 STRESS(stress_fstat);
+STRESS(stress_full);
 STRESS(stress_futex);
 STRESS(stress_get);
 STRESS(stress_getrandom);
@@ -2386,7 +2663,8 @@ STRESS(stress_heapsort);
 STRESS(stress_hsearch);
 STRESS(stress_icache);
 STRESS(stress_inotify);
-STRESS(stress_iosync);
+STRESS(stress_io);
+STRESS(stress_ioprio);
 STRESS(stress_itimer);
 STRESS(stress_kcmp);
 STRESS(stress_key);
@@ -2395,9 +2673,12 @@ STRESS(stress_klog);
 STRESS(stress_lease);
 STRESS(stress_link);
 STRESS(stress_lockbus);
+STRESS(stress_locka);
 STRESS(stress_lockf);
+STRESS(stress_lockofd);
 STRESS(stress_longjmp);
 STRESS(stress_lsearch);
+STRESS(stress_madvise);
 STRESS(stress_malloc);
 STRESS(stress_matrix);
 STRESS(stress_membarrier);
@@ -2412,12 +2693,14 @@ STRESS(stress_mmapfork);
 STRESS(stress_mmapmany);
 STRESS(stress_mremap);
 STRESS(stress_msg);
+STRESS(stress_msync);
 STRESS(stress_mq);
 STRESS(stress_nice);
 STRESS(stress_noop);
 STRESS(stress_null);
 STRESS(stress_numa);
 STRESS(stress_oom_pipe);
+STRESS(stress_opcode);
 STRESS(stress_open);
 STRESS(stress_personality);
 STRESS(stress_pipe);
@@ -2425,20 +2708,24 @@ STRESS(stress_poll);
 STRESS(stress_procfs);
 STRESS(stress_pthread);
 STRESS(stress_ptrace);
+STRESS(stress_pty);
 STRESS(stress_qsort);
 STRESS(stress_quota);
 STRESS(stress_rdrand);
 STRESS(stress_readahead);
-STRESS(stress_remap_file_pages);
+STRESS(stress_remap);
 STRESS(stress_rename);
 STRESS(stress_rlimit);
+STRESS(stress_rtc);
+STRESS(stress_sctp);
+STRESS(stress_seal);
 STRESS(stress_seccomp);
 STRESS(stress_seek);
-STRESS(stress_sem_posix);
+STRESS(stress_sem);
 STRESS(stress_sem_sysv);
-STRESS(stress_shm_posix);
-STRESS(stress_shm_sysv);
 STRESS(stress_sendfile);
+STRESS(stress_shm);
+STRESS(stress_shm_sysv);
 STRESS(stress_sigfd);
 STRESS(stress_sigfpe);
 STRESS(stress_sigpending);
@@ -2446,12 +2733,13 @@ STRESS(stress_sigsegv);
 STRESS(stress_sigsuspend);
 STRESS(stress_sigq);
 STRESS(stress_sleep);
-STRESS(stress_socket);
-STRESS(stress_socket_fd);
-STRESS(stress_socket_pair);
+STRESS(stress_sock);
+STRESS(stress_sockfd);
+STRESS(stress_sockpair);
 STRESS(stress_spawn);
 STRESS(stress_splice);
 STRESS(stress_stack);
+STRESS(stress_stackmmap);
 STRESS(stress_str);
 STRESS(stress_stream);
 STRESS(stress_switch);
@@ -2462,6 +2750,7 @@ STRESS(stress_sysfs);
 STRESS(stress_tee);
 STRESS(stress_timer);
 STRESS(stress_timerfd);
+STRESS(stress_tlb_shootdown);
 STRESS(stress_tsc);
 STRESS(stress_tsearch);
 STRESS(stress_udp);
