@@ -24,6 +24,10 @@
  */
 #define _GNU_SOURCE
 
+#include "stress-ng.h"
+
+#if defined(STRESS_SOCKET_FD)
+
 #include <stdio.h>
 #include <stdlib.h>
 #include <stdint.h>
@@ -40,7 +44,6 @@
 #include <arpa/inet.h>
 #include <sys/un.h>
 
-#include "stress-ng.h"
 
 #define MSG_ID			'M'
 #define MAX_FDS			(65536)
@@ -170,7 +173,7 @@ retry:
 
 		stress_set_sockaddr(name, instance, ppid,
 			AF_UNIX, opt_socket_fd_port,
-			&addr, &addr_len);
+			&addr, &addr_len, NET_ADDR_ANY);
 		if (connect(fd, addr, addr_len) < 0) {
 			(void)close(fd);
 			usleep(10000);
@@ -253,7 +256,8 @@ static int stress_socket_server(
 	}
 
 	stress_set_sockaddr(name, instance, ppid,
-		AF_UNIX, opt_socket_fd_port, &addr, &addr_len);
+		AF_UNIX, opt_socket_fd_port,
+		&addr, &addr_len, NET_ADDR_ANY);
 	if (bind(fd, addr, addr_len) < 0) {
 		rc = exit_status(errno);
 		pr_fail_dbg(name, "bind");
@@ -301,10 +305,10 @@ die:
 }
 
 /*
- *  stress_socket_fd
+ *  stress_sockfd
  *	stress socket fd passing
  */
-int stress_socket_fd(
+int stress_sockfd(
 	uint64_t *const counter,
 	const uint32_t instance,
 	const uint64_t max_ops,
@@ -329,3 +333,5 @@ again:
 		return stress_socket_server(counter, instance, max_ops, name, pid, ppid, max_fd);
 	}
 }
+
+#endif
