@@ -72,9 +72,15 @@ typedef struct {
 	int	   opt;
 } socket_opts_t;
 
+typedef struct {
+	const char *typename;
+	int	   type;
+} socket_type_t;
+
 static int opt_socket_domain = AF_INET;
 static int opt_socket_port = DEFAULT_SOCKET_PORT;
 static int opt_socket_opts = SOCKET_OPT_SEND;
+static int opt_socket_type = SOCK_STREAM;
 
 static const socket_opts_t socket_opts[] = {
 	{ "send",	SOCKET_OPT_SEND },
@@ -85,9 +91,19 @@ static const socket_opts_t socket_opts[] = {
 	{ NULL,		0 }
 };
 
+static const socket_type_t socket_type[] = {
+#if defined(SOCK_STREAM)
+	{ "stream",	SOCK_STREAM  },
+#endif
+#if defined(SOCK_SEQPACKET)
+	{ "seqpacket",	SOCK_SEQPACKET },
+#endif
+	{ NULL,		0 }
+};
+
 /*
  *  stress_set_socket_opts()
- *	parse --send-opts
+ *	parse --sock-opts
  */
 int stress_set_socket_opts(const char *optarg)
 {
@@ -100,7 +116,7 @@ int stress_set_socket_opts(const char *optarg)
 		}
 	}
 	fprintf(stderr, "sock-opts option '%s' not known, options are:", optarg);
-	for (i = 0; socket_opts[i].opt; i++) {
+	for (i = 0; socket_opts[i].optname; i++) {
 		fprintf(stderr, "%s %s",
 			i == 0 ? "" : ",", socket_opts[i].optname);
 	}
@@ -108,6 +124,29 @@ int stress_set_socket_opts(const char *optarg)
 	return -1;
 }
 
+/*
+ *  stress_set_socket_type()
+ *	parse --sock-type
+ */
+int stress_set_socket_type(const char *optarg)
+{
+	int i;
+
+	for (i = 0; socket_type[i].typename; i++) {
+		if (!strcmp(optarg, socket_type[i].typename)) {
+			opt_socket_type = socket_type[i].type;
+			return 0;
+		}
+	}
+	fprintf(stderr, "sock-type option '%s' not known, options are:", optarg);
+	for (i = 0; socket_type[i].typename; i++) {
+		fprintf(stderr, "%s %s",
+			i == 0 ? "" : ",", socket_type[i].typename);
+	}
+	fprintf(stderr, "\n");
+	return -1;
+}
+
 /*
  *  stress_set_socket_port()
  *	set port to use
@@ -130,10 +169,10 @@ int stress_set_socket_domain(const char *name)
 }
 
 /*
- *  stress_socket_client()
+ *  stress_sctp_client()
  *	client reader
  */
-static void stress_socket_client(
+static void stress_sctp_client(
 	uint64_t *const counter,
 	const uint32_t instance,
 	const uint64_t max_ops,
@@ -155,7 +194,7 @@ retry:
 			(void)kill(getppid(), SIGALRM);
 			exit(EXIT_FAILURE);
 		}
-		if ((fd = socket(opt_socket_domain, SOCK_STREAM, 0)) < 0) {
+		if ((fd = socket(opt_socket_domain, opt_socket_type, 0)) < 0) {
 			pr_fail_dbg(name, "socket");
 			/* failed, kick parent to finish */
 			(void)kill(getppid(), SIGALRM);
@@ -164,7 +203,7 @@ retry:
 
 		stress_set_sockaddr(name, instance, ppid,
 			opt_socket_domain, opt_socket_port,
-			&addr, &addr_len);
+			&addr, &addr_len, NET_ADDR_ANY);
 		if (connect(fd, addr, addr_len) < 0) {
 			(void)close(fd);
 			usleep(10000);
@@ -205,7 +244,7 @@ retry:
 /*
  *  handle_socket_sigalrm()
  *     catch SIGALRM
- *  stress_socket_client()
+ *  stress_sctp_client()
  *     client reader
  */
 static void MLOCKED handle_socket_sigalrm(int dummy)
@@ -215,10 +254,10 @@ static void MLOCKED handle_socket_sigalrm(int dummy)
 }
 
 /*
- *  stress_socket_server()
+ *  stress_sctp_server()
  *	server writer
  */
-static int stress_socket_server(
+static int stress_sctp_server(
 	uint64_t *const counter,
 	const uint32_t instance,
 	const uint64_t max_ops,
@@ -240,7 +279,7 @@ static int stress_socket_server(
 		rc = EXIT_FAILURE;
 		goto die;
 	}
-	if ((fd = socket(opt_socket_domain, SOCK_STREAM, 0)) < 0) {
+	if ((fd = socket(opt_socket_domain, opt_socket_type, 0)) < 0) {
 		rc = exit_status(errno);
 		pr_fail_dbg(name, "socket");
 		goto die;
@@ -253,7 +292,8 @@ static int stress_socket_server(
 	}
 
 	stress_set_sockaddr(name, instance, ppid,
-		opt_socket_domain, opt_socket_port, &addr, &addr_len);
+		opt_socket_domain, opt_socket_port,
+		&addr, &addr_len, NET_ADDR_ANY);
 	if (bind(fd, addr, addr_len) < 0) {
 		rc = exit_status(errno);
 		pr_fail_dbg(name, "bind");
@@ -281,7 +321,6 @@ static int stress_socket_server(
 #if defined(SOCKET_NODELAY)
 			int one = 1;
 #endif
-
 			len = sizeof(addr);
 			if (getsockname(fd, &addr, &len) < 0) {
 				pr_fail_dbg(name, "getsockname");
@@ -294,14 +333,15 @@ static int stress_socket_server(
 				(void)close(sfd);
 				break;
 			}
+
 #if defined(SOCKET_NODELAY)
 			if (opt_flags & OPT_FLAGS_SOCKET_NODELAY) {
 				if (setsockopt(fd, SOL_TCP, TCP_NODELAY, &one, sizeof(one)) < 0) {
 					pr_inf(stderr, "%s: setsockopt TCP_NODELAY "
 						"failed and disabled, errno=%d (%s)\n",
 						name, errno, strerror(errno));
-					opt_flags &= ~OPT_FLAGS_SOCKET_NODELAY;
-				}
+						opt_flags &= ~OPT_FLAGS_SOCKET_NODELAY;
+					}
 			}
 #endif
 			memset(buf, 'A' + (*counter % 26), sizeof(buf));
@@ -383,10 +423,10 @@ die:
 }
 
 /*
- *  stress_socket
+ *  stress_sock
  *	stress by heavy socket I/O
  */
-int stress_socket(
+int stress_sock(
 	uint64_t *const counter,
 	const uint32_t instance,
 	const uint64_t max_ops,
@@ -405,9 +445,9 @@ again:
 		pr_fail_dbg(name, "fork");
 		return EXIT_FAILURE;
 	} else if (pid == 0) {
-		stress_socket_client(counter, instance, max_ops, name, ppid);
+		stress_sctp_client(counter, instance, max_ops, name, ppid);
 		exit(EXIT_SUCCESS);
 	} else {
-		return stress_socket_server(counter, instance, max_ops, name, pid, ppid);
+		return stress_sctp_server(counter, instance, max_ops, name, pid, ppid);
 	}
 }
