@@ -48,16 +48,20 @@ static sigset_t set;
 typedef struct ctxt {
 	const char *name;
 	const char *path;
+	char *badbuf;
 } ctxt_t;
 
 /*
  *  stress_sys_read()
  *	read a proc file
  */
-static inline void stress_sys_read(const char *name, const char *path)
+static inline void stress_sys_read(
+	const char *name,
+	const char *path,
+	char *badbuf)
 {
 	int fd;
-	ssize_t i = 0;
+	ssize_t i = 0, ret;
 	char buffer[SYS_BUF_SZ];
 
 	if ((fd = open(path, O_RDONLY | O_NONBLOCK)) < 0)
@@ -67,7 +71,7 @@ static inline void stress_sys_read(const char *name, const char *path)
 	 *  Multiple randomly sized reads
 	 */
 	while (i < (4096 * SYS_BUF_SZ)) {
-		ssize_t ret, sz = 1 + (mwc32() % sizeof(buffer));
+		ssize_t sz = 1 + (mwc32() % sizeof(buffer));
 redo:
 		if (!opt_do_run)
 			break;
@@ -81,15 +85,49 @@ redo:
 			break;
 		i += sz;
 	}
+
+	/* file stat should be OK if we've just opened it */
+	if (opt_flags & OPT_FLAGS_VERIFY) {
+		struct stat buf;
+
+		if (fstat(fd, &buf) < 0) {
+			pr_fail(stderr, "%s: stat failed on %s: "
+				"errno=%d (%s)\n",
+				name, path, errno, strerror(errno));
+		} else {
+			if ((buf.st_mode & S_IROTH) == 0) {
+				pr_fail(stderr, "%s: read access failed on %s which "
+					"could be opened, errno=%d (%s)\n",
+				name, path, errno, strerror(errno));
+			}
+		}
+	}
 	(void)close(fd);
 
-	/* file should be R_OK if we've just opened it */
-	if ((access(path, R_OK) < 0) &&
-	    (opt_flags & OPT_FLAGS_VERIFY)) {
-		pr_fail(stderr, "%s: R_OK access failed on %s which "
-			"could be opened, errno=%d (%s)\n",
-			name, path, errno, strerror(errno));
+	if ((fd = open(path, O_RDONLY | O_NONBLOCK)) < 0)
+		return;
+	/*
+	 *  Zero sized reads
+	 */
+redo_zero:
+	ret = read(fd, buffer, 0);
+	if (ret < 0) {
+		if ((errno == EAGAIN) || (errno == EINTR))
+			goto redo_zero;
 	}
+
+	/*
+	 *  Bad read buffer
+	 */
+	if (badbuf) {
+redo_badbuf:
+		ret = read(fd, badbuf, SYS_BUF_SZ);
+		if (ret < 0) {
+			if ((errno == EAGAIN) || (errno == EINTR))
+				goto redo_badbuf;
+		}
+	}
+	(void)close(fd);
 }
 
 /*
@@ -100,7 +138,7 @@ redo:
 static void *stress_sys_read_thread(void *ctxt_ptr)
 {
 	static void *nowt = NULL;
-	uint8_t stack[SIGSTKSZ];
+	uint8_t stack[SIGSTKSZ] = { 0 };
         stack_t ss;
 	ctxt_t *ctxt = (ctxt_t *)ctxt_ptr;
 
@@ -124,7 +162,7 @@ static void *stress_sys_read_thread(void *ctxt_ptr)
 		return &nowt;
 	}
 	while (keep_running && opt_do_run)
-		stress_sys_read(ctxt->name, ctxt->path);
+		stress_sys_read(ctxt->name, ctxt->path, ctxt->badbuf);
 
 	return &nowt;
 }
@@ -145,6 +183,11 @@ static void stress_sys_read_threads(const char *name, const char *path)
 
 	memset(ret, 0, sizeof(ret));
 
+	ctxt.badbuf = mmap(NULL, SYS_BUF_SZ, PROT_READ,
+			MAP_SHARED | MAP_ANONYMOUS, -1, 0);
+	if (ctxt.badbuf == MAP_FAILED)
+		ctxt.badbuf = NULL;
+
 	keep_running = true;
 
 	for (i = 0; i < MAX_READ_THREADS; i++) {
@@ -154,7 +197,7 @@ static void stress_sys_read_threads(const char *name, const char *path)
 	for (i = 0; i < 8; i++) {
 		if (!opt_do_run)
 			break;
-		stress_sys_read(name, path);
+		stress_sys_read(name, path, ctxt.badbuf);
 	}
 	keep_running = false;
 
@@ -162,6 +205,9 @@ static void stress_sys_read_threads(const char *name, const char *path)
 		if (ret[i] == 0)
 			pthread_join(pthreads[i], NULL);
 	}
+
+	if (ctxt.badbuf)
+		munmap(ctxt.badbuf, SYS_BUF_SZ);
 }
 
 /*
@@ -234,7 +280,7 @@ int stress_sysfs(
 
 	(void)instance;
 
-	if (geteuid() == 0) {
+	if ((instance == 0) && (geteuid() == 0)) {
 		pr_inf(stderr, "%s: running as root, just traversing /sys "
 			"and not reading files.\n", name);
 		sys_read = false;
