@@ -32,7 +32,10 @@
 #include <stdio.h>
 #include <stdint.h>
 #include <stdlib.h>
+#include <stdbool.h>
 #include <unistd.h>
+#include <fcntl.h>
+#include <poll.h>
 #include <sys/syscall.h>
 #include <sys/mman.h>
 #include <sys/ioctl.h>
@@ -108,21 +111,34 @@ static inline int handle_page_fault(
 	uint8_t *data_end,
 	size_t page_size)
 {
-	struct uffdio_copy copy;
-
 	if ((addr < data_start) || (addr >= data_end)) {
 		pr_fail_err(name, "userfaultfd page fault address out of range");
 		return -1;
 	}
-	copy.copy = 0;
-	copy.mode = 0;
-	copy.dst = (unsigned long)addr;
-	copy.src = (unsigned long)zero_page;
-	copy.len = page_size;
 
-	if (ioctl(fd, UFFDIO_COPY, &copy) < 0) {
-		pr_fail_err(name, "userfaultfd page fault copy ioctl failed");
-		return -1;
+	if (mwc32() & 1) {
+		struct uffdio_copy copy;
+
+		copy.copy = 0;
+		copy.mode = 0;
+		copy.dst = (unsigned long)addr;
+		copy.src = (unsigned long)zero_page;
+		copy.len = page_size;
+
+		if (ioctl(fd, UFFDIO_COPY, &copy) < 0) {
+			pr_fail_err(name, "userfaultfd page fault copy ioctl failed");
+			return -1;
+		}
+	} else {
+		struct uffdio_zeropage zeropage;
+
+		zeropage.range.start = (unsigned long)addr;
+		zeropage.range.len = page_size;
+		zeropage.mode = 0;
+		if (ioctl(fd, UFFDIO_ZEROPAGE, &zeropage) < 0) {
+			pr_fail_err(name, "userfaultfd page fault zeropage ioctl failed");
+			return -1;
+		}
 	}
 	return 0;
 }
@@ -143,10 +159,12 @@ int stress_userfaultfd(
 	void *zero_page = NULL;
 	int fd, status, rc = EXIT_SUCCESS;
 	const unsigned int uffdio_copy = 1 << _UFFDIO_COPY;
+	const unsigned int uffdio_zeropage = 1 << _UFFDIO_ZEROPAGE;
 	pid_t pid;
 	struct uffdio_api api;
 	struct uffdio_register reg;
 	context_t c;
+	bool do_poll = true;
 
 	/* Child clone stack */
 	static uint8_t stack[STACK_SIZE];
@@ -186,6 +204,9 @@ int stress_userfaultfd(
 		goto unmap_data;
 	}
 
+	if (stress_set_nonblock(fd) < 0)
+		do_poll = false;
+
 	/* API sanity check */
 	memset(&api, 0, sizeof(api));
 	api.api = UFFD_API;
@@ -222,6 +243,13 @@ int stress_userfaultfd(
 		rc = EXIT_FAILURE;
 		goto unmap_data;
 	}
+	/* OK, so do we have zeropage supported? */
+	if ((reg.ioctls & uffdio_zeropage) != uffdio_zeropage) {
+		pr_err(stderr, "%s: ioctl UFFDIO_REGISTER did not support _UFFDIO_ZEROPAGE\n",
+			name);
+		rc = EXIT_FAILURE;
+		goto unmap_data;
+	}
 
 	/* Set up context for child */
 	c.name = name;
@@ -236,7 +264,7 @@ int stress_userfaultfd(
 	 *  We need to clone the child and share the same VM address space
 	 *  as parent so we can perform the page fault handling
 	 */
-	pid = clone(stress_userfaultfd_child, stack_top, 
+	pid = clone(stress_userfaultfd_child, align_stack(stack_top),
 		SIGCHLD | CLONE_FILES | CLONE_FS | CLONE_SIGHAND | CLONE_VM, &c);
 	if (pid < 0) {
 		pr_err(stderr, "%s: fork failed, errno = %d (%s)\n",
@@ -248,11 +276,47 @@ int stress_userfaultfd(
 	do {
 		struct uffd_msg msg;
 		ssize_t ret;
-	
+
 		/* check we should break out before we block on the read */
 		if (!opt_do_run)
 			break;
 
+		/*
+		 * polled wait exercises userfaultfd_poll
+		 * in the kernel, but only works if fd is NONBLOCKing
+		 */
+		if (do_poll) {
+			struct pollfd fds[1];
+
+			memset(fds, 0, sizeof fds);
+			fds[0].fd = fd;
+			fds[0].events = POLLIN;
+			/* wait for 1 second max */
+
+			ret = poll(fds, 1, 1000);
+			if (ret == 0)
+				continue;	/* timed out, redo the poll */
+			if (ret < 0) {
+				if (errno == EINTR)
+					continue;
+				if (errno != ENOMEM) {
+					pr_fail_err(name, "poll userfaultfd");
+					if (!opt_do_run)
+						break;
+				}
+				/*
+				 *  poll ran out of free space for internal
+				 *  fd tables, so give up and block on the
+				 *  read anyway
+				 */
+				goto do_read;
+			}
+			/* No data, re-poll */
+			if (!(fds[0].revents & POLLIN))
+				continue;
+		}
+
+do_read:
 		if ((ret = read(fd, &msg, sizeof(msg))) < 0) {
 			if (errno == EINTR)
 				continue;
@@ -265,10 +329,10 @@ int stress_userfaultfd(
 		if (msg.event != UFFD_EVENT_PAGEFAULT) {
 			pr_fail_err(name, "userfaultfd msg not pagefault event");
 			continue;
-		} 
+		}
 		/* We only expect a write fault */
 		if (!(msg.arg.pagefault.flags & UFFD_PAGEFAULT_FLAG_WRITE)) {
-			pr_fail_err(name, "userfaultfd msg not write pahge fault event");
+			pr_fail_err(name, "userfaultfd msg not write page fault event");
 			continue;
 		}
 		/* Go handle the page fault */
