@@ -68,6 +68,9 @@ int stress_utime(
 
 	do {
 		struct timeval times[2], *t;
+#if defined(__linux__)
+		struct timespec ts;
+#endif
 
 		if (gettimeofday(&times[0], NULL) < 0) {
 			t = NULL;
@@ -86,6 +89,22 @@ int stress_utime(
 				name, errno, strerror(errno));
 			break;
 		}
+
+		ts.tv_sec = UTIME_NOW;
+		ts.tv_nsec = UTIME_NOW;
+		if (futimens(fd, &ts) < 0) {
+			pr_dbg(stderr, "%s: futimens failed: errno=%d: (%s)\n",
+				name, errno, strerror(errno));
+			break;
+		}
+
+		ts.tv_sec = UTIME_OMIT;
+		ts.tv_nsec = UTIME_OMIT;
+		if (futimens(fd, &ts) < 0) {
+			pr_dbg(stderr, "%s: futimens failed: errno=%d: (%s)\n",
+				name, errno, strerror(errno));
+			break;
+		}
 #endif
 		if (utime(filename, NULL) < 0) {
 			pr_dbg(stderr, "%s: utime failed: errno=%d: (%s)\n",
