@@ -955,7 +955,7 @@ static size_t stress_vm_flip(
 {
 	const size_t chunk_sz = sizeof(uint8_t) * 8;
 	volatile uint8_t *ptr;
-	uint8_t *buf_end = buf + sz, bit = 0b00000011;
+	uint8_t *buf_end = buf + sz, bit = 0x03;
 	uint64_t w, z, c = *counter;
 	size_t bit_errors = 0, i;
 
@@ -1934,6 +1934,7 @@ again:
 				instance);
 			/* If we got killed by OOM killer, re-start */
 			if (WTERMSIG(status) == SIGKILL) {
+				log_system_mem_info();
 				pr_dbg(stderr, "%s: assuming killed by OOM killer, "
 					"restarting again (instance %d)\n",
 					name, instance);
@@ -1959,8 +1960,10 @@ again:
 			if (!keep || (buf == NULL)) {
 				if (!opt_do_run)
 					return EXIT_SUCCESS;
-				buf = mmap(NULL, buf_sz, PROT_READ | PROT_WRITE,
-					MAP_SHARED | MAP_ANONYMOUS | opt_vm_flags, -1, 0);
+				buf = (uint8_t *)mmap(NULL, buf_sz,
+					PROT_READ | PROT_WRITE,
+					MAP_SHARED | MAP_ANONYMOUS |
+					opt_vm_flags, -1, 0);
 				if (buf == MAP_FAILED) {
 					buf = NULL;
 					no_mem_retries++;
@@ -1984,12 +1987,12 @@ again:
 
 			if (!keep) {
 				(void)madvise_random(buf, buf_sz);
-				(void)munmap(buf, buf_sz);
+				(void)munmap((void *)buf, buf_sz);
 			}
 		} while (opt_do_run && (!max_ops || *counter < max_ops));
 
 		if (keep && buf != NULL)
-			(void)munmap(buf, buf_sz);
+			(void)munmap((void *)buf, buf_sz);
 	}
 	*counter >>= VM_BOGO_SHIFT;
 
