@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2013-2015 Canonical, Ltd.
+ * Copyright (C) 2013-2016 Canonical, Ltd.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -133,9 +133,7 @@ int tz_get_temperatures(tz_info_t **tz_info_list, stress_tz_t *tz)
 	for (tz_info = *tz_info_list; tz_info; tz_info = tz_info->next) {
 		char path[PATH_MAX];
 		FILE *fp;
-		size_t i;
-
-		i = tz_info->index;
+		size_t i = tz_info->index;
 
 		snprintf(path, sizeof(path),
 			"/sys/class/thermal/%s/temp",
@@ -143,8 +141,10 @@ int tz_get_temperatures(tz_info_t **tz_info_list, stress_tz_t *tz)
 
 		tz->tz_stat[i].temperature = 0;
 		if ((fp = fopen(path, "r")) != NULL) {
-			if (fscanf(fp, "%" SCNu64, &tz->tz_stat[i].temperature) != 1)
+			if (fscanf(fp, "%" SCNu64,
+			     &tz->tz_stat[i].temperature) != 1) {
 				tz->tz_stat[i].temperature = 0;
+			}
 			fclose(fp);
 		}
 	}
@@ -194,10 +194,13 @@ void tz_dump(
 				if (!dumped_heading) {
 					dumped_heading = true;
 					pr_inf(stdout, "%s:\n", munged);
-					pr_yaml(yaml, "    - stressor: %s\n", munged);
+					pr_yaml(yaml, "    - stressor: %s\n",
+					munged);
 				}
-				pr_inf(stdout, "%20s %7.2f °C\n", tz_info->type, temp);
-				pr_yaml(yaml, "      %s: %7.2f\n", tz_info->type, temp);
+				pr_inf(stdout, "%20s %7.2f °C\n",
+					tz_info->type, temp);
+				pr_yaml(yaml, "      %s: %7.2f\n",
+					tz_info->type, temp);
 				no_tz_stats = false;
 			}
 		}
